@@ -1,8 +1,6 @@
 /* Lisp eval
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2001 Scott "Jerry" Lawrence.
-   Copyright (c) 2005, 2006, 2007 Reuben Thomas.
+   Copyright (c) 2001, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -28,7 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 
@@ -46,8 +44,8 @@ struct fentry
 typedef struct fentry fentry;
 
 static fentry fentry_table[] = {
-#define X(zile_name, c_name, doc)               \
-  {zile_name, F_ ## c_name, true, doc},
+#define X(zile_name, c_name, interactive, doc)   \
+  {zile_name, F_ ## c_name, interactive, doc},
 #include "tbl_funcs.h"
 #undef X
 };
@@ -124,7 +122,7 @@ evaluateBranch (le * trybranch)
   func = get_fentry (keyword->data);
   leWipe (keyword);
   if (func)
-    return func->func (0, trybranch);
+    return func->func (1, trybranch);
 
   return NULL;
 }
@@ -153,7 +151,7 @@ evaluateNode (le * node)
   return value;
 }
 
-DEFUN ("setq", setq)
+DEFUN_HIDDEN ("setq", setq)
 /*+
 (setq [sym val]...)
 
@@ -236,7 +234,6 @@ DEFUN ("execute-extended-command", execute_extended_command)
 Read function name, then read its arguments and call it.
 +*/
 {
-  le * ret;
   const char *name;
   astr msg = astr_new ();
 
@@ -254,10 +251,8 @@ Read function name, then read its arguments and call it.
   if (name == NULL)
     return false;
 
-  ret = execute_function (name, uniarg);
+  ok = execute_function (name, uniarg);
   free ((char *) name);
-
-  return ret;
 }
 END_DEFUN
 
@@ -280,7 +275,7 @@ minibuf_read_function_name (const char *fmt, ...)
   add_macros_to_list (cp->completions, completion_strcmp);
 
   va_start (ap, fmt);
-  ms = vminibuf_read_completion (fmt, "", cp, functions_history,
+  ms = minibuf_vread_completion (fmt, "", cp, functions_history,
                                  "No function name given",
                                  minibuf_test_in_completions,
                                  "Undefined function name `%s'", ap);
@@ -294,7 +289,6 @@ void
 init_eval (void)
 {
   functions_history = history_new ();
-  get_fentry ("setq")->interactive = false;
 }
 
 void
