@@ -92,11 +92,10 @@ bool
 expand_path (astr path)
 {
   int ok = true;
-  struct passwd *pw;
   const char *sp = astr_cstr (path), *p;
   astr epath = astr_new ();
 
-  if (*sp != '/')
+  if (*sp != '/' && *sp != '~')
     {
       astr cwd = agetcwd ();
       astr_cat (epath, cwd);
@@ -122,6 +121,8 @@ expand_path (astr path)
         }
       else if (*p == '~' && (p == sp || p[-1] == '/'))
         { /* Got `/~' or leading `~'.  Restart from this point. */
+          struct passwd *pw;
+
           astr_truncate (epath, 0);
           ++p;
 
@@ -164,8 +165,9 @@ expand_path (astr path)
           p += 2;
         }
 
-      while (*p != '\0' && *p != '/')
-        astr_cat_char (epath, *p++);
+      if (*p != '~')
+        while (*p != '\0' && *p != '/')
+          astr_cat_char (epath, *p++);
     }
 
   astr_cpy (path, epath);
@@ -1242,14 +1244,15 @@ zile_exit (int doabort)
   for (bp = head_bp; bp != NULL; bp = bp->next)
     if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
       {
-        astr buf;
-        buf = astr_new ();
+        astr buf = astr_new (), as;
         if (bp->filename != NULL)
           astr_cpy_cstr (buf, bp->filename);
         else
           astr_cpy_cstr (buf, bp->name);
-        /* FIXME: Generate ZILE in next line */
-        astr_cat_cstr (buf, ".ZILESAVE");
+        as = astr_new_cstr (PACKAGE);
+        astr_recase (as, UPPERCASE);
+        astr_afmt (buf, ".%sSAVE", astr_cstr (as));
+        astr_delete (as);
         fprintf (stderr, "Saving %s...\r\n", astr_cstr (buf));
         raw_write_to_disk (bp, astr_cstr (buf), S_IRUSR | S_IWUSR);
         astr_delete (buf);
