@@ -1,7 +1,6 @@
 /* Point facility functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2004 David A. Capello.
+   Copyright (c) 2004, 2008 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -22,7 +21,7 @@
 
 #include "config.h"
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 Point
